#include "MPU9250.h"
#include "Integrator.h"

MPU9250 imu(Wire, 0x68);
INS ins;
unsigned long prevTime;

void setup()
{
  Serial.begin(115200);

  int status = imu.begin();
  if (status < 0)
  {
    Serial.print("IMU Error: ");
    Serial.println(status);
    while (1) { }
  }

  imu.readSensor();
  ins.setGravity(imu.getAccelX_mss(), imu.getAccelY_mss(), imu.getAccelZ_mss());
}

void loop()
{
  unsigned long nowTime = micros();
  unsigned long deltaTime = nowTime - prevTime;
  float dt = deltaTime / 1000000.0f;

  imu.readSensor();
  ins.accumulate(imu.getAccelX_mss(), imu.getAccelY_mss(), imu.getAccelZ_mss(),
    imu.getGyroX_rads(), imu.getGyroY_rads(), imu.getGyroZ_rads(), dt);

  if (millis() % 200 == 0)
  {
    Serial.print("dt : ");
    Serial.print(deltaTime);
    Serial.print("\t");
    Serial.println(dt, 10);
    Serial.print("ins :\t");
    Serial.print(ins.posX());
    Serial.print("\t");
    Serial.print(ins.posY());
    Serial.print("\t");
    Serial.print(ins.posZ());
    Serial.print("\t");
    Serial.print(ins.rotX());
    Serial.print("\t");
    Serial.print(ins.rotY());
    Serial.print("\t");
    Serial.println(ins.rotZ());
    Serial.print("spd :\t");
    Serial.print(ins.posDX());
    Serial.print("\t");
    Serial.print(ins.posDY());
    Serial.print("\t");
    Serial.print(ins.posDZ());
    Serial.print("\t");
    Serial.print(ins.rotDX());
    Serial.print("\t");
    Serial.print(ins.rotDY());
    Serial.print("\t");
    Serial.println(ins.rotDZ());
    Serial.print("grv :\t");
    Serial.print(ins.grvX());
    Serial.print("\t");
    Serial.print(ins.grvY());
    Serial.print("\t");
    Serial.print(ins.grvZ());
    Serial.print("\t(len: ");
    Serial.print(sqrt(ins.grvX()*ins.grvX()+ins.grvY()*ins.grvY()+ins.grvZ()*ins.grvZ()));
    Serial.println(")");
    Serial.print("imu :\t");
    Serial.print(imu.getAccelX_mss());
    Serial.print("\t");
    Serial.print(imu.getAccelY_mss());
    Serial.print("\t");
    Serial.print(imu.getAccelZ_mss());
    Serial.print("\t");
    Serial.print(imu.getGyroX_rads());
    Serial.print("\t");
    Serial.print(imu.getGyroY_rads());
    Serial.print("\t");
    Serial.println(imu.getGyroZ_rads());
    Serial.print("img :\t");
    Serial.print(imu.getAccelX_mss() - ins.grvX());
    Serial.print("\t");
    Serial.print(imu.getAccelY_mss() - ins.grvY());
    Serial.print("\t");
    Serial.println(imu.getAccelZ_mss() - ins.grvZ());
  }

  prevTime = nowTime;
}
